/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

if (typeof HT == 'undefined') {
    var HT = {};
}

HT.taskListInterval = 60000;

HT.Core = (function() {
    /*
     * Human task UI Core which manage all the modules.
     * Core tell modules when they should initialize and when they should stop.
     * Core enables inter-module communication.
     */

    var moduleData = {}, // Holds the currently registered modules
            currentTasks = {}; // Holds the current task list(work as a cache)

    return {

        /**
         * Register new module in core.
         * @param moduleId  ID of the module
         * @param module    Module object(already initialized)
         */
        registerModule: function(moduleId, module) {
            moduleData[moduleId] = module;
        },

        addTaskToCache: function(taskId, task) {
            currentTasks[taskId] = task;
        },

        clearTaskCache: function() {
            currentTasks = {};
        },

        getTaskFromCache: function(taskId) {
            return currentTasks[taskId];
        },

        /**
         * Start a module specified by Id using init method of the module
         * @param moduleId module to start
         */
        start: function(moduleId) {
            if (moduleData[moduleId]) {
                moduleData[moduleId].init()
            }
        },

        /**
         * Stop a module by calling module's destroy method.
         * @param moduleId module to destroy
         */
        stop: function(moduleId) {
            if (moduleData[moduleId]) {
                moduleData[moduleId].destroy();
                moduleData[moduleId] = null;
            }
        },

        /**
         * Init the application core. This method will contain all initialization logic.
         */
        init: function() {
            this.start("task-list");
            this.start("pagination-widget");
            this.start("task-view");
            this.start("dialog-widget");
            //this.start("button-bar");
        },

        /**
         * Perform an asynchronous HTTP (Ajax) request.
         * @param url              A string containing the URL to which the request is sent.
         * @param data             Data to be sent to the server.
         * @param dataType         The type of data that you're expecting back from the server.
         * @param successCallback  A function to be called if the request succeeds.
         * @param failureCallback  A function to be called if the request fails.
         */
        ajax: function(url, data, dataType, successCallback, failureCallback) {
            $.ajax({
                url: url,
                type: 'GET',
                dataType: dataType,
                data: data,
                success: successCallback,
                error: failureCallback
            });
        },

        /**
         * Add the given comment to the specified task.
         * @param taskId Task to add the comment
         * @param comment comment text
         * @param success callback function to execute when operation successful
         * @param failure callback function to execute when operation fail
         */
        addCommentToTask: function(taskId, comment, success, failure) {
            this.ajax("task-operations-ajaxprocessor.jsp",
                    {operation: "addComment", task: taskId, commentValue: comment},
                    "json",
                    success,
                    failure);
        },

        /**
         * Claim the responsibility of the task. Task to reserved state.
         * @param taskId Task Id
         */
        claimTask: function(taskId, success, failure) {
            this.ajax("task-operations-ajaxprocessor.jsp",
                    {operation: "claim", task: taskId},
                    "json",
                    success,
                    failure);
        },

        /**
         * Start the task, move the task to InProgress state
         * @param taskId Task Id
         */
        startTask: function(taskId, success, failure) {
             this.ajax("task-operations-ajaxprocessor.jsp",
                    {operation: "start", task: taskId},
                    "json",
                    success,
                    failure);
        },

        /**
         * Stop the task, move the task to Reserved state
         * @param taskId Task Id
         */
        stopTask: function(taskId, success, failure) {
             this.ajax("task-operations-ajaxprocessor.jsp",
                    {operation: "stop", task: taskId},
                    "json",
                    success,
                    failure);
        },

        /**
         * Release the task, send the task back to the ready state
         * @param taskId Task ID
         */
        releaseTask: function(taskId, success, failure) {
             this.ajax("task-operations-ajaxprocessor.jsp",
                    {operation: "release", task: taskId},
                    "json",
                    success,
                    failure);
        },

        getCommentsFromTask: function(taskId, success, failure) {
            this.ajax("comments_list-ajaxprocessor.jsp",
                    {task: taskId},
                    "json",
                    success,
                    failure);
        }
    };

}());

HT.Sandbox = (function(core) {
    /**
     * The Sandbox ensures a consistence interface. Acts like a security guard.
     * Sandbox knows what the modules are allowed to access and do on the framework.
     */
    var listeners = {};

    return {
        /**
         * Notify all the listeners registered for certain type of event based
         * on the event data.
         * @param eventData  JSON object which follows the below format.
         *  eventData = {type: ...., data: ....};
         */
        notify: function(eventData) {
            // Consider about adding typeOf to check whether event listeners return array for given event type.
            var listenersForEvent = listeners[eventData.type];
            if(listenersForEvent != undefined){
                for (var i = 0; i < listenersForEvent.length; i++) {
                    var listener = listenersForEvent[i];
                    if (typeof listener === 'function') {
                        listener(eventData.data);
                    }
                }
            }
        },

        /**
         * Modules can register their own listeners in Sandbox. When events occurred
         * Sandbox will notify those listeners.
         * @param event    Interested event type.
         * @param listener Listener function
         * @param contextForListener Listener context (used to handle closure)
         */
        listen: function(event, listener, contextForListener) {
            if (listeners[event] === undefined) {
                listeners[event] = new Array();
            }

            listeners[event].push(listener);

        },

        /**
         * Calls services which returns JSON
         * @param url service URL.
         * @param data Data to be sent to the service
         * @param success callback function if call is successful
         * @param failure callback function if call fails
         */
        getJSON: function(url, data, success, failure) {
            // TODO: Should check for types
            core.ajax(url, data, 'json', success, failure)
        },

        /**
         * Clear the task cache maintained in core.
         */
        clearTaskCache: function() {
            core.clearTaskCache();
        },

        /**
         * Add task to the task cache in core.
         * @param taskId  ID of the task
         * @param task    Task object
         */
        addTaskToCache: function(taskId, task) {
            core.addTaskToCache(taskId, task);
        },

        /**
         * Get task from the cache.
         * @param taskId
         */
        getTask: function(taskId) {
            return core.getTaskFromCache(taskId);
        },

        claimTask: function(taskId, success, failure) {
            return core.claimTask(taskId, success, failure);
        },

        startTask: function(taskId, success, failure) {
            return core.startTask(taskId, success, failure);
        },

        stopTask: function(taskId, success, failure) {
            return core.stopTask(taskId, success, failure);
        },

        releaseTask: function(taskId, success, failure) {
            return core.releaseTask(taskId, success, failure);
        },

        addCommentToTask: function(taskId, comment, success, failure) {
            return core.addCommentToTask(taskId, comment, success, failure);
        },

        getCommentsForTask: function(taskId, success, failure) {
            return core.getCommentsFromTask(taskId, success, failure);
        }
    };
}(HT.Core));

HT.Core.registerModule("task-list", (function(sandbox) {
    /**
     * Task List Module. This module manage the task list. It handles
     * task list loading, populating task list table and making tasks selectable.
     * Task List module notify other about task availability, and task selections.
     * And it listen to paginator events and some other events.
     */
    var tasks = null,              // currently loaded tasks
            selected = null,       // currently selected task
            currentPage = 0,       // current page of task list(use for pagination)
            pages = 0,             // number pages
            genericHumanRole = '', // Generic human role filter
            workQueue = '',        // Work queue filter
            whereClause = '',      // where clause for task filtering
            createdOnClause = '',  // created on clause for task filterng
            taskType = 'ALL',      // type of tasks (ALL, NOTIFICATION, TASK)
            loadTaskListIntervalObject = null, // interval object of task list loader
            taskListAjaxProcessorRelativeURL = "task_list-ajaxprocessor.jsp", // task list request handler URL

            taskListTableHTMLStr = '';  // HTML table string which going to append to HTML UI as task List

    function getTaskListOperationFailed(xmlHttpRequest, errorType, exception) {
        $("#taskListContainer").html("<p>Error occurred:" + errorType + "</p>");
        clearInterval(loadTaskListIntervalObject);
    }

    function handleErrorFromServer(errorObject, message) {
        if (errorObject != null) {
            $("#taskListContainer").html("<p>" + errorObject.error + "</p>");
        } else {
            $("#taskListContainer").html("<p>" + message + "</p>");
        }
        clearInterval(loadTaskListIntervalObject);
    }

    function handleEmptyTaskList() {
        $("#taskListContainer").html("<p>No tasks available.</p>");
    }

    function populateTaskTableHeader() {
        taskListTableHTMLStr += "<table id='taskList' class='styledLeft'><thead><tr><th>ID</th><th>Status</th>" +
                                "<th>Name</th><th>Subject</th><th>Priority</th><th>Created On</th><th>Due On</th>" +
                                "</tr></thead><tbody id='taskListTableBody'>";
    }

    function addTaskToTable(task) {
        taskListTableHTMLStr += "<tr class='taskRow' id='tr" + task.id + "'><td>" + task.id + "</td><td>" + task.status + "</td><td>" + task.name +
                                "</td><td>" + task.subject + "</td><td>" + task.priority + "</td><td>" +
                                task.created + "</td><td>" + task.due + "</td></tr>";
    }

    function populateTaskTableFooter() {
        taskListTableHTMLStr += "<tr><td id='paginationControls' colspan='7'><div id='paginationButtonBar'></div></td></tr></tbody>";
    }

    function makeTableSelectable(callback) {
        $("#taskListTableBody .taskRow").hover(function() {
            if (!$(this).hasClass('ui-selected')) {
                $(this).toggleClass('ui-hover');
            }
            var id = $(this).attr('id');
            $("#taskListTableBody .taskRow").each(function() {
                if (id != $(this).attr('id') && $(this).hasClass('ui-hover') == true) {
                    $(this).removeClass('ui-hover');
                }
            });
        });
        $("#taskListTableBody .taskRow").click(function() {
            // Get the row id to use when enabling required actions and
            // performing actions given by user like claiming the task.
            var id = $(this).attr('id');
            var taskId = id.substring(2);

            // When mouse enters to this row element 'hover' event will color(#EEEFFB) it.
            // When user click on the row we need to remove that color and color the row using selected color(#B4D5F0).
            if ($(this).hasClass('ui-hover') == true) {
                $(this).removeClass('ui-hover');
            }

            // To enable/disable buttons and setting the selected task, we need to
            // identify whether this click is a selection or de-selection.
            if (!($(this).hasClass('ui-selected'))) {
                // This means this is a new select of a table row.
                // We need to enable
                // To use when reloading the list. The previously selected row will be selected even
                // after reloading.
                // TODO: implement reload handling
                selected = id;
                sandbox.notify({type: 'task-selected', data:taskId});
            } else {
                // If ui-selected class is already there this means deselection.
                sandbox.notify({type: 'task-deselected', data:taskId});
            }

            // Set the selected color based on selection/de-selection
            $(this).toggleClass('ui-selected');


            $("#taskListTableBody .taskRow").each(function() {
                if (id != $(this).attr('id') && $(this).hasClass('ui-selected') == true) {
                    $(this).removeClass('ui-selected');
                }
            });
        });
    }

    function addTasksToTable(tasks) {
        for (var i = 0; i < tasks.length; i++) {
            var task = tasks[i];
            // Adding task information to cache
            sandbox.addTaskToCache(task.id, task);
            addTaskToTable(task);
        }
    }

    function fillTaskList(tasksList, textStatus, xmlHttpRequest) {
        // Notifying the listeners about task list loading.
        // Mainly used to clear the task view when updating.
        // If there was a selected task, it will handle at the end of this function.
        sandbox.notify({type: "task-list-loaded", data: null});

        // Clearing the task list cache before loading new tasks.
        sandbox.clearTaskCache();

        taskListTableHTMLStr = "";
        if (tasksList == null) {
            handleErrorFromServer(null, "Error occurred during request to the server. Please check your connection.");
            clearInterval(loadTaskListIntervalObject);
        } else if (tasksList.error) {
            handleErrorFromServer(tasksList, "Error returned from server.");
            clearInterval(loadTaskListIntervalObject);
        } else if (tasksList.pages == 0) {
            handleEmptyTaskList();
            clearInterval(loadTaskListIntervalObject);
        } else {
            // Add Task Table's Header section
            populateTaskTableHeader();

            // Add tasks to table
            addTasksToTable(tasksList.tasks);

            // Add Table Footer
            populateTaskTableFooter();
            $("#taskListContainer").html(taskListTableHTMLStr);

            // Make task rows selectable
            makeTableSelectable(null);

            // When operation is performed on task view we need to reload the task list.
            // At reload we need to select the previously selected task again for usability improvements
            // So what we are doing here is simulate task select event manually.
            handleTaskSelectedBeforeReloadEvent();

            pages = tasksList.pages;

            // Notify the listeners about task list loading.
            sandbox.notify({type: "tasks-loaded", data: tasksList});
        }
    }

    function handleTaskSelectedBeforeReloadEvent(){
        // If the selected is not null
        if(selected){
            // If task with selected ID is available on the current task cache
            if(sandbox.getTask(selected.substring(2))){
                var taskRowSelector = "#" + selected;
                var taskId = selected.substring(2);
                // Add the ui-selected class to task row
                $(taskRowSelector).toggleClass('ui-selected');
                // Notify task-selected event to make task view available.
                sandbox.notify({type: 'task-selected', data:taskId});
            }
        }
    }

    function loadTaskList() {
        var data = {
            page: currentPage,
            taskType: taskType,
            genericHumanRole: genericHumanRole,
            workQueue: workQueue,
            whereClause: whereClause,
            createdOnClause: createdOnClause
        };

        sandbox.getJSON(
                taskListAjaxProcessorRelativeURL,
                data,
                fillTaskList,
                getTaskListOperationFailed);
    }

    function toFirstPage() {
        currentPage = 0;
        clearAndLoad();
    }

    function toPrevPage() {
        currentPage--;
        clearAndLoad();
    }

    function toNextPage() {
        currentPage++;
        clearAndLoad();
    }

    function toLastPage() {
        currentPage = pages - 1;
        clearAndLoad();
    }

    function clearAndLoad() {
        // Clear the previously added interval object.
        clearInterval(loadTaskListIntervalObject);

        // Load the task list again.
        // TODO: In operation-performed event we should only load that task. But currently we use load whole list
        loadTaskList();
        
        // create the interval object again
        setInterval(loadTaskList, HT.taskListInterval);
    }

    return {
        init: function() {
            // Load the task list
            loadTaskList();
            // Create interval object to periodically update task list
            loadTaskListIntervalObject = setInterval(loadTaskList, HT.taskListInterval);

            // Task List module interested about following events.
            sandbox.listen("first-clicked", toFirstPage, this);
            sandbox.listen("prev-clicked", toPrevPage, this);
            sandbox.listen("next-clicked", toNextPage, this);
            sandbox.listen("last-clicked", toLastPage, this);
            sandbox.listen("operation-performed", clearAndLoad, this);
        },

        destroy: function() {
            clearInterval(loadTaskListIntervalObject);
        }
    };
}(HT.Sandbox)));


HT.Core.registerModule("pagination-widget", (function(sandbox) {
    var currentPage = -1,
            pages = 0;

    function handleTasksLoadedEvent(tasksList) {
        currentPage = tasksList.currentPage;
        pages = tasksList.pages;

        // If current page equal 1, no prev, no first
        // If current page equal page numbers, no next, no last
        var paginationButtonsHtml = "<button id='paginationFirst'>First</button>" +
                                    "<button id='paginationPrev'>Previous</button>" +
                                    "<span style='font-size:10px;'>Page " + currentPage + " of " + pages + "</span>" +
                                    "<button id='paginationNext'>Next</button>" +
                                    "<button id='paginationLast'>Last</button>";
        $("#paginationButtonBar").html(paginationButtonsHtml);
        if (pages === 1) {
            makeButtons(true, true, true, true);
        } else if (currentPage === 1) {
            makeButtons(true, true, false, false);
        } else if (currentPage === pages) {
            makeButtons(false, false, true, true);
        } else {
            makeButtons(false, false, false, false);
        }

    }

    function onFirstButtonClick() {
        sandbox.notify({type: "first-clicked", data: null});
    }

    function onPrevButtonClick() {
        sandbox.notify({type: "prev-clicked", data: null});
    }

    function onNextButtonClick() {
        sandbox.notify({type: "next-clicked", data: null});
    }

    function onLastButtonClick() {
        sandbox.notify({type: "last-clicked", data: null});
    }

    function makeButtons(first, previous, next, last) {
        makeButton('#paginationFirst', 'ui-icon-seek-first', first, onFirstButtonClick);
        makeButton('#paginationPrev', 'ui-icon-seek-prev', previous, onPrevButtonClick);
        makeButton('#paginationNext', 'ui-icon-seek-next', next, onNextButtonClick);
        makeButton('#paginationLast', 'ui-icon-seek-end', last, onLastButtonClick);
    }

    function makeButton(id, icon, disabled, onClick) {
        $(id).click(onClick).button({
            icons: {
                primary: icon
            },
            text: false,
            disabled: disabled
        });
    }

    return {
        init: function() {
            sandbox.listen("tasks-loaded", handleTasksLoadedEvent, this);
        }
    };
}(HT.Sandbox)));


HT.Core.registerModule("task-view", (function(sandbox){
    /**
     * Module whih handles the task view. This module is listening to task-selection events
     * and show the correct task according to user selection.
     */
    var currentTask = null;

    function handleResponse(response){
        if(response !== undefined){
            if(response.result){
                sandbox.notify({
                    type: "show-dialog",
                    data: {
                        type:"INFO",
                        message: response.result,
                        yesCallback: null
                    }});
            }else if(response.error){
                sandbox.notify({
                    type: "show-dialog",
                    data: {
                        type:"INFO",
                        message: response.error,
                        yesCallback: null
                    }});
            }
        }
    }
    /**
     * Event handler which handles claim button click event.
     */
    function onClaimClick(){
        function claimSuccess(response, textStatus, xmlHttpRequest){
            handleResponse(response);
            sandbox.notify({type:"operation-performed", data: null});
        }

        function claimFailure(xmlHttpRequest, errorType, exception){
            sandbox.notify({
                type: "show-dialog",
                data: {
                    type:"INFO",
                    message: "Error claiming task " + currentTask,
                    yesCallback: null
                }});

        }


        function claimTask(){
            sandbox.claimTask(currentTask, claimSuccess, claimFailure);
        }

        sandbox.notify({
            type: "show-dialog",
            data: {
                type: "CONFIRMATION",
                message: "Do you want to claim task " + currentTask,
                yesCallback: claimTask,
                noCallback: null
            }});
    }

    /**
     * Event handler which handles start button click event.
     */
    function onStartClick(){
        // TODO: In this callback we need to keep the selected task as it is and reload the task list and task view
        function startSuccess(response, textStatus, xmlHttpRequest){
            handleResponse(response);
            sandbox.notify({type:"operation-performed", data: null});
        }

        function startFailure(xmlHttpRequest, errorType, exception){
             sandbox.notify({
                type: "show-dialog",
                data: {
                    type:"INFO",
                    message: "Error starting task " + currentTask,
                    yesCallback: null
                }});
        }
        function startTask(){
            sandbox.startTask(currentTask, startSuccess, startFailure);
        }

        sandbox.notify({
            type: "show-dialog",
            data: {
                type: "CONFIRMATION",
                message: "Do you want to start task " + currentTask,
                yesCallback: startTask,
                noCallback: null
            }});
    }

    /**
     * Event handler which handles stop button click event.
     */
    function onStopClick(){
        // TODO: In this callback we need to keep the selected task as it is and reload the task list and task view
        function stopSuccess(response, textStatus, xmlHttpRequest){
            handleResponse(response);
            sandbox.notify({type:"operation-performed", data: null});
        }

        function stopFailure(xmlHttpRequest, errorType, exception){
             sandbox.notify({
                type: "show-dialog",
                data: {
                    type:"INFO",
                    message: "Error stoping task " + currentTask,
                    yesCallback: null
                }});
        }
        function stopTask(){
            sandbox.stopTask(currentTask, stopSuccess, stopFailure);
        }

        sandbox.notify({
            type: "show-dialog",
            data: {
                type: "CONFIRMATION",
                message: "Do you want to stop task " + currentTask,
                yesCallback: stopTask,
                noCallback: null
            }});
    }

    /**
     * Event handler which handled release button click event.
     */
    function onReleaseClick(){
        function releaseSuccess(response, textStatus, xmlHttpRequest){
            handleResponse(response);
            sandbox.notify({type:"operation-performed", data: null});
        }

        function releaseFailure(xmlHttpRequest, errorType, exception){
            sandbox.notify({
                type: "show-dialog",
                data: {
                    type:"INFO",
                    message: "Error releasing task " + currentTask,
                    yesCallback: null
                }});
        }

        function releaseTask(){
            sandbox.releaseTask(currentTask, releaseSuccess, releaseFailure);
        }

        sandbox.notify({
            type: "show-dialog",
            data: {
                type: "CONFIRMATION",
                message: "Do you want to release task " + currentTask,
                yesCallback: releaseTask,
                noCallback: null
            }});
    }

    /**
     * Event handler which handles add comment button click event.
     */
    function onAddCommentClick(){
        // TODO: In this callback we need to keep the selected task as it is and reload the task list and task view

        var comment = "";
        function addCommentSuccess(response, textStatus, xmlHttpRequest){
            handleResponse(response);
            sandbox.notify({type:"operation-performed", data: null});
        }

        function addCommentFailure(xmlHttpRequest, errorType, exception){
            sandbox.notify({
                type: "show-dialog",
                data: {
                    type:"INFO",
                    message: "Error adding comment to the task " + currentTask,
                    yesCallback: null
                }});
        }

        function addComment(){
            comment = $("#commentText").val();
            sandbox.addCommentToTask(currentTask, comment, addCommentSuccess, addCommentFailure);
        }


        sandbox.notify({
            type: "show-dialog",
            data: {
                type: "CONFIRMATION",
                message: "Do you want to add a comment to the task " + currentTask,
                yesCallback: addComment,
                noCallback: null
            }});
    }


    /**
     * populate the comments list for current
     * @param taskId
     */
    function populateComments(taskId)  {

        /**
         * Populate comments to task view
         * @param comments  JSON object array.
         */
        function addCommentsToTaskView(comments){
            var commentListHtml = "<ul>"
            var commentCount = comments.length;
            for(var i = 0; i < commentCount; i++){
                commentListHtml += "<li>" + comments[i].text +
                                   "<br/><span class='commentAuthorTime'>Commented by " +
                                   comments[i].addedBy + " at " + comments[i].addedAt + ".";
            }
            
            commentListHtml += "</ul>";
            $("#commentList").html(commentListHtml);
        }

        function getCommentsSuccess(response, textStatus, xmlHttpRequest){
            if(response !== undefined){
                if(response.comments){
                    addCommentsToTaskView(response.comments);
                }else if (response.message){
                    $("#commentList").html("<p>" + response.message + "</p>");
                }else if(response.error){
                    $("#commentList").html("<p>" + response.error + "</p>");                    
                }
            }
        }

        function getCommentsFailed(xmlHttpRequest, errorType, exception){
            sandbox.notify({
                type: "show-dialog",
                data: {
                    type:"INFO",
                    message: "Error loading comments for task " + currentTask,
                    yesCallback: null
                }});
        }

        sandbox.getCommentsForTask(currentTask, getCommentsSuccess, getCommentsFailed);
    }

    /**
     * Handle task selection event
     * @param taskId selected task
     */
    function handleTaskSelection(taskId){
        currentTask = taskId;
        var task = sandbox.getTask(taskId);
        if(task){
            createTaskView(task);
        }                                
        loadTaskRequestContent(task);
        loadTaskResponseContent(task);
    }

    function loadTaskResponseContent(task) {
        if (task.isCompletable) {
            var taskResponseContentURL = "task_tryit-ajaxprocessor.jsp";
            jQuery("#taskResponseContent").load(taskResponseContentURL,
                                                {taskId: task.id, tryitServiceName: task.responseService ,
                                                    tryitOperationName: task.responseOperation},
                                                function(resText, status, res) {
                                                    if (status != "success") {
                                                        CARBON.showErrorDialog("Error loading response content");
                                                    }
                                                }
                    );
        } else if (task.status == "COMPLETED") {
            var taskResponsedContentURL = "task_mockit-ajaxprocessor.jsp";
            jQuery("#taskResponseContent").load(taskResponsedContentURL,
                                            {taskId: task.id, type: "RESPONSE"},
                                            function(resText, status, res) {
                                                if (status != "success") {
                                                    CARBON.showErrorDialog("Error loading responsed content");
                                                }
                                            }
                );
        }
    }

    function loadTaskRequestContent(task) {
        var taskRequestContentURL = "task_mockit-ajaxprocessor.jsp";
        jQuery("#taskRequestContent").load(taskRequestContentURL,
                                            {taskId: task.id, type: "REQUEST"},
                                            function(resText, status, res) {
                                                if (status != "success") {
                                                    CARBON.showErrorDialog("Error loading request content");
                                                }
                                            }
                );
    }

    function handleTaskDeSelection(taskId){
        currentTask = null;
        clearTaskView();
        
    }

    /**
     * Decorate HTML buttons using jQuery UI buttons and
     * add click event handlers.
     */
//    function decorateButtons(isClaimEnabled, isReleaseEnabled, isStartenabled, isCommentsEnabled){
//        $("#claimTaskButton").button({disabled: !isClaimEnabled}).click(onClaimClick);
//        $("#releaseTaskButton").button({disabled: !isReleaseEnabled}).click(onReleaseClick);
//        $("#startTaskButton").button({disabled: !isStartenabled}).click(onStartClick);
//        $("#addCommentButton").button({disabled: !isCommentsEnabled}).click(onAddCommentClick);
//    }

    /**
     * Decorate HTML buttons using jQuery UI buttons and
     * add click event handlers.
     */
    function decorateButtons(task){
        $("#claimTaskButton").button({disabled: !task.isClaimable}).click(onClaimClick);
        $("#releaseTaskButton").button({disabled: !task.isReleasable}).click(onReleaseClick);
        $("#startTaskButton").button({disabled: !task.isStartable}).click(onStartClick);
        $("#stopTaskButton").button({disabled: !task.isStopable}).click(onStopClick);
        $("#addCommentButton").button({disabled: (task.type != "TASK" || task.status == "COMPLETED")}).
                click(onAddCommentClick);
    }

    function decorateTabs(){
        $('#tvAdditionalInfoTab').tabs({ fx: { height: 'toggle', opacity: 'toggle' } });
    }

    /**
     * Clear the task view div by adding child div which used as a workaround to a
     * issue in DOM creation plugin.
     */
    function clearTaskView(){
        $("#taskView").html("<div id='workAroundToDOMCreationIssue'/>");
    }

    /**
     * Create the HTML DOM for Task View Module.
     * Uses jquery.createdomnodes plugin.
     * @param task
     */
    function createTaskView(task){
        var owner = "No Owner";
        clearTaskView();
        var context = $("#workAroundToDOMCreationIssue");

        if(task.status == "RESERVED" ||
                task.status == "IN_PROGRESS" ||
                task.status == "SUSPENDED" ||
                task.status == "COMPLETED" ||
                task.status == "FAILED"){
            if(task.owner)
                owner = task.owner;
        }

        $(context)
        ._table()
            .addClass("styledLeft")
            .attr("id", "taskViewTable")
            ._thead()
                ._tr()
                    ._th()
                        .addClass("tvTableHeader")
                        .attr("colspan", "2")
                        ._span_()
                            .attr("style", "float:left;padding-top:3px !important;")
                            .text(task.name)
                        ._span()
                            .attr("style", "float:right; font-size:9px;padding-right:5px !important;padding-top:3px !important; padding-bottom:3px !important;")
                            .attr("id", "buttonspan")
                            ._button_()
                                .attr("id", "claimTaskButton")
                                .text("Claim")
                            ._button_()
                                .attr("id", "releaseTaskButton")
                                .text("Release")
                            ._button_()
                                .attr("id", "startTaskButton")
                                .text("Start")
                            ._button_()
                                .attr("id", "stopTaskButton")
                                .text("Stop")
                        .span_()
                    .th_()
                .tr_()
            .thead_()
            ._tbody()
                ._tr()
                    ._td_()
                        .text("Subject")
                    ._td_()
                        .text(task.subject)
                .tr_()
                ._tr()
                    ._td_()
                        .text("Status")
                    ._td_()
                        .text(task.status)
                .tr_()
                ._tr()
                    ._td_()
                        .text("Priority")
                    ._td_()
                        .text(task.priority)
                .tr_()
                ._tr()
                    ._td_()
                        .text("Owner")
                    ._td_()
                        .text(owner)
                .tr_()
                ._tr()
                    ._td_()
                        .text("Created On")
                    ._td_()
                        .text(task.created)
                .tr_()
                ._tr()
                    ._td()
                        .attr("style", "padding-left:1px !important;")
                        .attr("colspan", "2")
                        ._table()
                            .addClass("noBorders")
                            .attr("style", "border:none;width:100%;")
                            ._tbody()
                                ._tr()
                                    ._td()
                                        .attr("id", "taskRequestHeader")
                                        .attr("style", "border-bottom:1px solid #CCCCCC;padding-left:0px !important;")
                                        ._span_()
                                            .addClass("tv_underline_header")
                                            .text("Request")
                                    .td_()
                                .tr_()
                                ._tr()
                                    ._td_()
                                        .attr("id", "taskRequestContent")
                                        .attr("style", "padding-left:0px !important;")
                                        .text("Request goes here")
                                .tr_()
                                ._tr()
                                    ._td()
                                        .attr("id", "taskResponseHeader")
                                        .attr("style", "border-bottom:1px solid #CCCCCC;padding-left:0px !important;")
                                        ._span_()
                                            .addClass("tv_underline_header")
                                            .text("Response")
                                    .td_()
                                .tr_()
                                ._tr()
                                    ._td_()
                                        .attr("id", "taskResponseContent")
                                        .attr("style", "padding-left:0px !important;")
                                        .text("Request goes here")
                                .tr_()
                                ._tr()
                                    ._td()
                                        .attr("style", "padding-left:0px !important;")
                                        ._div()
                                            .attr("id", "tvAdditionalInfoTab")
                                            .attr("style", "float:none;")
                                            ._ul()
                                                ._li()
                                                    ._a_()
                                                        .attr("href", "#tv_comments")
                                                        .text("Comments")
                                                .li_()
                                                ._li()
                                                    ._a_()
                                                        .attr("href", "#tv_attachments")
                                                        .text("Attachments")
                                                .li_()
                                            .ul_()
                                            ._div()
                                                .addClass("tv_tabdiv")
                                                .attr("id", "tv_comments")
                                                ._textarea_()
                                                    .attr("id", "commentText")
                                                    .attr("style", "width:100%;")
                                                ._button_()
                                                    .attr("id", "addCommentButton")
                                                    .text("Add Comment")
                                                ._div_()
                                                    .attr("id", "commentList")
                                            .div_()
                                            ._div_()
                                                .addClass("tv_tabdiv")
                                                .attr("id", "tv_attachments")
                                                .text("Attachments not supported in this version.")
                                    .td_()
                                .tr_()
                            .tbody_()
                        .table_()
                    .td_()
                .tr_()
            .tbody_()
        .table_()                  

        // Decorate buttons using jQuery UI Button plugin.
        decorateButtons(task);
        if (!task.isCompletable && task.status != "COMPLETED") {
            $("#taskResponseContent").css("display", "none");
            $("#taskResponseHeader").css("display", "none");
        }
        if (task.type == "NOTIFICATION") {
            $("#buttonspan").css("display", "none");
        }


        // Make comments and attachments tabs using jQuery UI Tab plugin.
        decorateTabs();

        // Populate comments tab
        populateComments();
    }
    
    return {
        init: function(){
            // At task selection we need to show the complete view of task
            sandbox.listen("task-selected", handleTaskSelection, this);
            // At task de-selection we need to clear the task view
            sandbox.listen("task-deselected", handleTaskDeSelection, this);

            // When pagination buttons are clicked we need to clear the task view
            sandbox.listen("first-clicked", clearTaskView, this);
            sandbox.listen("prev-clicked", clearTaskView, this);
            sandbox.listen("next-clicked", clearTaskView, this);
            sandbox.listen("last-clicked", clearTaskView, this);

            // We need to clear the current selected task view when task list is re-loading,
            // and in the reloading logic it'll again show the selected task in task view.
            sandbox.listen("task-list-loaded", clearTaskView, this);
        }
    }           
}(HT.Sandbox)));

HT.Core.registerModule("dialog-widget", (function(sandbox){

    function handleShowDialog(eventData){
        if(eventData.type == "INFO"){
            showInfoDialog(eventData.message, eventData.yesCallback);
        }else if(eventData.type == "CONFIRMATION"){
            showConfirmationDialog(eventData.message, eventData.yesCallback, eventData.noCallback);
        }
    }

    function showInfoDialog(message, okCallback){
        $("#htDialog").html("<p>" + message + "</p>");

        $("#htDialog").dialog({
            modal: true,
            buttons:  {
                Ok: function(){
                    if(okCallback){
                        okCallback();
                        $(this).dialog('close');
                    } else {
                        $(this).dialog('close');
                    }
                }
            }
        });
    }

    function showConfirmationDialog(message, yesCallback, noCallback){
        $("#htDialog").html("<p>" + message + "</p>");

        $("#htDialog").dialog({
            modal: true,
            buttons:  {
                "Yes": function(){
                    if(yesCallback !== undefined && typeof yesCallback == "function"){
                        yesCallback();
                        $(this).dialog('close');
                    } else {
                        $(this).dialog('close');
                    }
                },

                "No": function(){
                    if(noCallback){
                        noCallback();
                    } else {
                        $(this).dialog('close');
                    }
                }
            }
        });
    }

    return {
        init: function(){
            // This module listen to show-dialog event and display the dialog widget using information
            // provided through event data.
            // Handle show dialog needs, dialog type, message and callback functions for YES, NO.
            // If only YES callback function is present we take it as a dialog with OK button.
            sandbox.listen("show-dialog", handleShowDialog, this);
        }
    }

}(HT.Sandbox)));

